<?php
/**
 * 访问类或对象成员
 * 1. 访问对象成员: '->'对象访问符
 * 2. 访问类成员:  '::' 范围解析符
 *
 * $this和self
 * 1. $this:引用当前类的实例(对象)
 * 2. self: 引用当前类
 *
 * 范围解析符使用场景:
 * 1. 类中: 引用本类的静态属性或方法 self::name; self::getName();
 * 2. 类外: 访问类中的静态成员: className::property; className::method();
 */

//类中使用对象的引用$this
class Demo1
{
    private $name;  //私有属性

    public function __construct($name='Peter')
    {
        $this->name = $name; //用$this访问属性
    }

    public function hello()
    {
        return 'Hello '.$this->getName(); //用$this访问属性
    }

    public function getName()
    {
        return $this->name;
    }

}

$obj1 = new Demo1();
echo $obj1->hello();  //外部用类的对象来访问
echo '<hr>';

//类中使用本类的引用self

/**
 * 静态成员的概念
 * 1. 静态成员为所有对象所享,不属性于具体对象,使用关键字:static声明
 * 2. 类常量也属于静态成员,使用关键字: const 声明
 * 3. 静态成员(属性和方法)必须要使用self::访问,属性名前要加$符号
 * 4. 静态方法中,只能引用静态属性
 * 5. 非静态方法也可以使用self::访问
 * 6. 类外访问静态成员或类常量必须使用类名,而不是类的引用self
 */
class Demo2
{
//    public static $lesson = 'PHP';
    private static $lesson = 'PHP';  //改为私有静态,外部不能访问
    const SITE_NAME = '中文网';  //创建类常量,常量必须在声明时初始化
    protected static $name = 'Peter Zhu';
    //创建一个外部接口方法来访问私有静态属性
    public function getLesson ()
    {
        return self::$lesson;  //在非静态方法中可以访问静态成员
    }
    //创建一个外部接口静态方法来访问私有静态属性
    public static function getLesson1 ()
    {
        return self::$lesson;  //在非静态方法中可以访问静态成员
    }

    public static function getSiteName()
    {
        return self::$lesson.self::SITE_NAME; //访问类常量
    }
}

class Demo3 extends Demo2  //继承自Demo2
{
    public static function getName ()
    {
        return self::$name;  //访问父类中的静态属性
    }
}


//echo Demo2::$lesson;
echo (new Demo2)->getLesson();  //使用对象来访问非静态方法
echo '<hr>';
echo Demo2::getLesson1(); //使用类名来访问表态方法
echo '<hr>';
echo Demo2::getSiteName();
echo '<hr>';
//echo Demo2::$name; //因为是受保护的,外部不可访问
echo Demo3::getName();  //在Demo2的子类中可以访问